import React, { useState, useEffect } from 'react';
import { useRouter } from 'next/router';
import { useFormik } from 'formik';
import * as Yup from 'yup';
import { toast } from 'react-hot-toast';
import { FaSave, FaTrash, FaTimes } from 'react-icons/fa';

import Button from '@/components/common/Button';
import Input from '@/components/common/Input';
import Select from '@/components/common/Select';
import ImageUploader from '@/components/admin/ImageUploader';
import RichTextEditor from '@/components/admin/RichTextEditor';

import { post, get, put } from '@/lib/api-client';
import { sanitizeHtml } from '@/lib/sanitize';
import { Product, Category } from '@/types/models';
import styles from '@/styles/admin/ProductForm.module.scss';

interface ProductFormProps {
  initialData?: Partial<Product>;
  isEdit?: boolean;
}

const ProductForm: React.FC<ProductFormProps> = ({ 
  initialData = {}, 
  isEdit = false 
}) => {
  const router = useRouter();
  const [categories, setCategories] = useState<Category[]>([]);
  const [loading, setLoading] = useState<boolean>(false);
  const [editorContent, setEditorContent] = useState<string>(initialData.description || '');
  
  // Fetch categories on mount
  useEffect(() => {
    const fetchCategories = async () => {
      try {
        const response = await get<Category[]>('/api/admin/categories');
        
        if (response.success && response.data) {
          setCategories(response.data);
        }
      } catch (error) {
        console.error('Error fetching categories:', error);
        toast.error('Không thể tải danh sách danh mục');
      }
    };
    
    fetchCategories();
  }, []);
  
  // Form validation schema
  const validationSchema = Yup.object({
    name: Yup.string()
      .required('Tên sản phẩm là bắt buộc')
      .max(100, 'Tên sản phẩm không được quá 100 ký tự'),
    slug: Yup.string()
      .required('Slug là bắt buộc')
      .matches(/^[a-z0-9-]+$/, 'Slug chỉ chứa các ký tự a-z, 0-9 và dấu gạch ngang')
      .max(100, 'Slug không được quá 100 ký tự'),
    price: Yup.number()
      .required('Giá là bắt buộc')
      .min(0, 'Giá không được âm'),
    salePrice: Yup.number()
      .nullable()
      .min(0, 'Giá khuyến mãi không được âm')
      .max(Yup.ref('price'), 'Giá khuyến mãi phải nhỏ hơn giá gốc'),
    categoryId: Yup.string().required('Danh mục là bắt buộc'),
    shortDescription: Yup.string()
      .required('Mô tả ngắn là bắt buộc')
      .max(250, 'Mô tả ngắn không được quá 250 ký tự'),
    sku: Yup.string().max(50, 'SKU không được quá 50 ký tự'),
    inStock: Yup.boolean(),
    image: Yup.string().required('Hình ảnh sản phẩm là bắt buộc'),
    features: Yup.array().of(Yup.string())
  });
  
  // Initialize form with Formik
  const formik = useFormik({
    initialValues: {
      name: initialData.name || '',
      slug: initialData.slug || '',
      price: initialData.price || 0,
      salePrice: initialData.salePrice || null,
      categoryId: initialData.category?._id || '',
      shortDescription: initialData.shortDescription || '',
      sku: initialData.sku || '',
      inStock: initialData.inStock !== undefined ? initialData.inStock : true,
      image: initialData.image || '',
      features: initialData.features || ['', '', '']
    },
    validationSchema,
    onSubmit: async (values) => {
      try {
        setLoading(true);
        
        // Create or update product payload
        const productData = {
          ...values,
          description: sanitizeHtml(editorContent), // Sanitize HTML before saving
        };
        
        // Handle empty features
        productData.features = productData.features.filter(f => f.trim() !== '');
        
        let response;
        
        if (isEdit) {
          // Update existing product
          response = await put<Product>(`/api/admin/products/${initialData._id}`, productData);
        } else {
          // Create new product
          response = await post<Product>('/api/admin/products', productData);
        }
        
        if (response.success) {
          toast.success(isEdit ? 'Cập nhật sản phẩm thành công' : 'Tạo sản phẩm thành công');
          router.push('/admin/products');
        } else {
          toast.error(response.message || 'Có lỗi xảy ra');
        }
      } catch (error) {
        console.error('Error saving product:', error);
        toast.error('Không thể lưu sản phẩm');
      } finally {
        setLoading(false);
      }
    }
  });
  
  // Generate slug from product name
  const generateSlug = () => {
    const slug = formik.values.name
      .toLowerCase()
      .replace(/à|á|ạ|ả|ã|â|ầ|ấ|ậ|ẩ|ẫ|ă|ằ|ắ|ặ|ẳ|ẵ/g, 'a')
      .replace(/è|é|ẹ|ẻ|ẽ|ê|ề|ế|ệ|ể|ễ/g, 'e')
      .replace(/ì|í|ị|ỉ|ĩ/g, 'i')
      .replace(/ò|ó|ọ|ỏ|õ|ô|ồ|ố|ộ|ổ|ỗ|ơ|ờ|ớ|ợ|ở|ỡ/g, 'o')
      .replace(/ù|ú|ụ|ủ|ũ|ư|ừ|ứ|ự|ử|ữ/g, 'u')
      .replace(/ỳ|ý|ỵ|ỷ|ỹ/g, 'y')
      .replace(/đ/g, 'd')
      .replace(/\s+/g, '-')
      .replace(/[^\w-]+/g, '')
      .replace(/--+/g, '-')
      .replace(/^-+|-+$/g, '');
      
    formik.setFieldValue('slug', slug);
  };
  
  // Add feature field
  const addFeature = () => {
    formik.setFieldValue('features', [...formik.values.features, '']);
  };
  
  // Remove feature field
  const removeFeature = (index: number) => {
    const newFeatures = [...formik.values.features];
    newFeatures.splice(index, 1);
    formik.setFieldValue('features', newFeatures);
  };
  
  return (
    <form onSubmit={formik.handleSubmit} className={styles.productForm}>
      <div className={styles.formGrid}>
        <div className={styles.formColumn}>
          <Input
            label="Tên sản phẩm"
            name="name"
            value={formik.values.name}
            onChange={formik.handleChange}
            onBlur={(e) => {
              formik.handleBlur(e);
              if (!isEdit && !formik.touched.slug) {
                generateSlug();
              }
            }}
            error={formik.touched.name && formik.errors.name}
            required
            fullWidth
          />
          
          <div className={styles.slugField}>
            <Input
              label="Slug"
              name="slug"
              value={formik.values.slug}
              onChange={formik.handleChange}
              onBlur={formik.handleBlur}
              error={formik.touched.slug && formik.errors.slug}
              required
              fullWidth
            />
            {!isEdit && (
              <Button
                type="button"
                variant="secondary"
                size="sm"
                onClick={generateSlug}
                className={styles.generateButton}
              >
                Tạo slug
              </Button>
            )}
          </div>
          
          <div className={styles.priceFields}>
            <Input
              label="Giá"
              name="price"
              type="number"
              value={formik.values.price.toString()}
              onChange={formik.handleChange}
              onBlur={formik.handleBlur}
              error={formik.touched.price && formik.errors.price}
              required
            />
            
            <Input
              label="Giá khuyến mãi"
              name="salePrice"
              type="number"
              value={formik.values.salePrice?.toString() || ''}
              onChange={formik.handleChange}
              onBlur={formik.handleBlur}
              error={formik.touched.salePrice && formik.errors.salePrice}
            />
          </div>
          
          <Select
            label="Danh mục"
            name="categoryId"
            value={formik.values.categoryId}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.categoryId && formik.errors.categoryId}
            required
            fullWidth
          >
            <option value="">Chọn danh mục</option>
            {categories.map((category) => (
              <option key={category._id} value={category._id}>
                {category.name}
              </option>
            ))}
          </Select>
          
          // Tiếp tục component ProductForm.tsx
          <Input
            label="SKU"
            name="sku"
            value={formik.values.sku}
            onChange={formik.handleChange}
            onBlur={formik.handleBlur}
            error={formik.touched.sku && formik.errors.sku}
          />
          
          <div className={styles.checkboxField}>
            <label>
              <input
                type="checkbox"
                name="inStock"
                checked={formik.values.inStock}
                onChange={formik.handleChange}
              />
              Còn hàng
            </label>
          </div>
          
          <div className={styles.imageUploader}>
            <label className={styles.imageLabel}>
              Hình ảnh sản phẩm <span className={styles.required}>*</span>
            </label>
            <ImageUploader
              currentImage={formik.values.image}
              onImageUploaded={(url) => formik.setFieldValue('image', url)}
              folder="products"
            />
            {formik.touched.image && formik.errors.image && (
              <div className={styles.errorText}>{formik.errors.image}</div>
            )}
          </div>
        </div>
        
        <div className={styles.formColumn}>
          <div className={styles.formField}>
            <label htmlFor="shortDescription" className={styles.label}>
              Mô tả ngắn <span className={styles.required}>*</span>
            </label>
            <textarea
              id="shortDescription"
              name="shortDescription"
              value={formik.values.shortDescription}
              onChange={formik.handleChange}
              onBlur={formik.handleBlur}
              rows={3}
              className={styles.textarea}
            />
            {formik.touched.shortDescription && formik.errors.shortDescription && (
              <div className={styles.errorText}>{formik.errors.shortDescription}</div>
            )}
          </div>
          
          <div className={styles.formField}>
            <label className={styles.label}>Mô tả chi tiết</label>
            <RichTextEditor
              content={editorContent}
              onChange={setEditorContent}
              placeholder="Nhập mô tả chi tiết sản phẩm..."
            />
          </div>
          
          <div className={styles.featuresSection}>
            <label className={styles.label}>Tính năng nổi bật</label>
            
            {formik.values.features.map((feature, index) => (
              <div key={index} className={styles.featureRow}>
                <Input
                  name={`features[${index}]`}
                  value={feature}
                  onChange={formik.handleChange}
                  placeholder={`Tính năng ${index + 1}`}
                  fullWidth
                />
                <Button
                  type="button"
                  variant="text"
                  icon={<FaTimes />}
                  onClick={() => removeFeature(index)}
                  aria-label="Xóa tính năng"
                  className={styles.removeFeatureButton}
                />
              </div>
            ))}
            
            <Button
              type="button"
              variant="outline"
              size="sm"
              onClick={addFeature}
              className={styles.addFeatureButton}
            >
              Thêm tính năng
            </Button>
          </div>
        </div>
      </div>
      
      <div className={styles.formActions}>
        <Button
          type="button"
          variant="outline"
          onClick={() => router.back()}
        >
          Hủy
        </Button>
        <Button
          type="submit"
          variant="primary"
          icon={<FaSave />}
          loading={loading}
          disabled={loading}
        >
          {isEdit ? 'Cập nhật sản phẩm' : 'Tạo sản phẩm'}
        </Button>
      </div>
    </form>
  );
};

export default ProductForm;